﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Customization;

using System.IO;
using System.Windows.Media.Imaging;

namespace CUIX_Webcast_Demos
{
    public class Class1
    {
        const string workspaceName = "Demo Workspace";
        
        Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
                   
        CustomizationSection cs;
        
        public Class1()
        {
            // retrieve the location of, and open the ACAD Main CUI File
            string mainCuiFile = (string)Application.GetSystemVariable("MENUNAME");
            mainCuiFile += ".cuix";
            cs = new CustomizationSection(mainCuiFile);
        }
        
        [CommandMethod("DEMOADDTAB")]
        public void demoAddTab()
        {
            //Get the RibbonRoot from the AutoCAD Menu
            RibbonRoot ribbonRoot = cs.MenuGroup.RibbonRoot;
            
            //Get the panels from the RibbonRoot
            RibbonPanelSourceCollection panels = ribbonRoot.RibbonPanelSources;

            RibbonPanelSource ribPanelSource = null;
            
            // Check to see if DemoPanel already exists
            foreach (RibbonPanelSource panel in panels)
            {
                if (panel.Text == "DemoPanel")
                {
                    ribPanelSource = panel;
                    ed.WriteMessage("\nDemoPanel already exists");
                    break;
                }
            }
           
            if (ribPanelSource == null)
            {
                //DemoPanel does not exist so create a new RibbonPanelSource
                ribPanelSource = new RibbonPanelSource(ribbonRoot);

                ribPanelSource.Text = "DemoPanel";

                //Create a RibbonRow
                RibbonRow ribRow = new RibbonRow();

                //Add the RibbonRow to the RibbonPanelSource
                ribPanelSource.Items.Add((RibbonItem)ribRow);

                //Create a Ribbon Command Button
                RibbonCommandButton ribCommandButton = createRibbonCmdButton(ribRow, "DemoButton", "ID_CircleRad", false);

                //Add the ribbon command button to the ribbon row
                ribRow.Items.Add((RibbonItem)ribCommandButton);

                // Create a Ribbon Command Button to add to the RibbonSplitButton 
                // created next. Create a new command
                RibbonCommandButton ribCommandButton2 = createRibbonCmdButton(ribRow, "DemoButton2", "ID_CircleRad", true);

                String strSplitButtonID = "DemoSplitButton_ID";
                string splitButtonName = "DemoSplitButton";
                RibbonSplitButton ribSplitButton;
                ribSplitButton = createRibbonSplitButton(ribRow, ribCommandButton2, splitButtonName, strSplitButtonID);

                //Add the ribbon split button to the ribbon row
                ribRow.Items.Add((RibbonItem)ribSplitButton);

                // Add the Ribbon Panel Source to the Ribbon Panels
                panels.Add(ribPanelSource);
            }

            RibbonTabSource tabSrc = null;
            // Check to see if DemoPanel already exists
            //ribbonRoot.FindTab(strElementId); Don't know the element id
            //Using name here for demo purposes. Use ElementId if possible.
            //There can be multiple tabs with the same value for text.
            RibbonTabSourceCollection ribTabSourceCol = ribbonRoot.RibbonTabSources;
            foreach (RibbonTabSource tabSource in ribTabSourceCol) //ribbonRoot.RibbonTabSources)
            {
                if (tabSource.Text == "DemoTab")
                {
                    tabSrc = tabSource;
                    ed.WriteMessage("\nDemoTab already exists");
                    break;
                }
            }
            
            if (tabSrc == null)
            {
                // Create a new Ribbon Tab Source
                tabSrc = new RibbonTabSource(ribbonRoot);
                tabSrc.Name = "DemoTab";
                tabSrc.Text = "DemoTab";

                //Add the Ribbon Tab Source to the collection
                RibbonTabSourceCollection tabs = null;
                tabs = ribbonRoot.RibbonTabSources;
                tabs.Add(tabSrc);

                //Add the Panel to the Tab
                RibbonPanelSourceReference ribPanelSourceRef = null;
                ribPanelSourceRef = new RibbonPanelSourceReference(tabSrc);
                ribPanelSourceRef.PanelId = ribPanelSource.ElementID;

                ribPanelSourceRef.ElementID = "DemoPanelSourceRefElementId";
                tabSrc.Items.Add(ribPanelSourceRef);

                //false parameter will not make a workspace current
                //after CUIx file is reloaded
                saveCui(false);
            }
        }
        RibbonCommandButton createRibbonCmdButton(RibbonRow ribRow, String buttonName, String strMacroID, bool bCreateNewCmd)
        {
            //Create a Ribbon Command Button
            RibbonCommandButton ribCommandButton = new RibbonCommandButton(ribRow);
            ribCommandButton.Text = buttonName;
            if (bCreateNewCmd)
            {
                MacroGroup macGroup;
                macGroup = cs.MenuGroup.MacroGroups[0];
                
                //This will add a new Command, 
                MenuMacro menuMac;
                
                //NOTE: If this is run again it creates another command with the same name.
                // A check should be done so that a different name is used if it is already
                // being used. 
                int indexOfMacro;
                //Need to change this to an ElementID of one of your macros
                indexOfMacro = macGroup.MenuMacros.IndexOf("MMU_0032");
                if (indexOfMacro == -1)
                {
                    //create MenuMacro as it does not exist
                    //Using an existing icons "RCDATA_16_3DALIGN", "RCDATA_32_3DALIGN"
                    menuMac = macGroup.CreateMenuMacro("DemoCommand", "^C^C_Line ", "ID_DemoCmd", "Demo line help",
                                     MacroType.Any, "RCDATA_16_3DALIGN", "RCDATA_32_3DALIGN", "Test command label");
                    ribCommandButton.MacroID = menuMac.ElementID;
                }
                else
                {
                    //MenuMacro already exists so just set the ElementID
                    ribCommandButton.MacroID = "MMU_0032"; 
                }
            }
            else
            {
                ribCommandButton.MacroID = strMacroID;

            }
            return ribCommandButton;
        }

        RibbonSplitButton createRibbonSplitButton(RibbonRow ribRow, RibbonCommandButton ribCmdButton, String splitButtonName, String strSplitButtonID)
        {
            //Create a Ribbon Command Button
            RibbonSplitButton ribSplitButton = new RibbonSplitButton(ribRow);
            ribSplitButton.Text = splitButtonName;
            ribSplitButton.ButtonStyle = RibbonButtonStyle.SmallWithoutText;
            ribSplitButton.ListStyle = RibbonSplitButtonListStyle.IconText;

            String SplitbuttonElementID = strSplitButtonID; // "mySplitButton_ID";
            ribSplitButton.ElementID = SplitbuttonElementID;

            RibbonSplitButtonBehavior rsbehavior = RibbonSplitButtonBehavior.DropDownFollow;
            ribSplitButton.Behavior = rsbehavior;

            RibbonSplitButtonListStyle rsStyle = RibbonSplitButtonListStyle.IconText;
            ribSplitButton.ListStyle = rsStyle;

            ribSplitButton.LargeImage = "";

            RibbonButtonStyle rsButtonStyle = RibbonButtonStyle.LargeWithHorizontalText;
            ribSplitButton.ButtonStyle = rsButtonStyle;

            string sKeyTip = "my Key Tip";
            ribSplitButton.KeyTip = sKeyTip;

            ribSplitButton.Items.Add(ribCmdButton);

            return ribSplitButton;
        }

        [CommandMethod("DEMOADDTOWS")]
        public void demoAddToWorkspace() 
        {
            // This command adds the tab to the Demo Workspace
            WSRibbonRoot wrkSpaceRibbonRoot = null;
            WorkspaceRibbonTabCollection wsRibbonTabCollection = null;
            WSRibbonTabSourceReference wsRibTabSrcRef = null;
            string strWrkSpaceName = "Demo Workspace";
            
            try
            {
                WorkspaceCollection WsCollect = cs.Workspaces;
                if ((WsCollect.Count <= 0))
                {
                    ed.WriteMessage("Failed to Get the WorkspaceCollection\n");
                    return;
                }

                foreach (Workspace wkSpace in WsCollect)
                {
                    if (wkSpace.Name == strWrkSpaceName)
                    {
                        wrkSpaceRibbonRoot = wkSpace.WorkspaceRibbonRoot;
                        break;
                    }

                }

                if (wrkSpaceRibbonRoot == null)
                {
                    //Demo Workspace does not exist need to create it
                    Workspace demoWS = new Workspace(cs, strWrkSpaceName);
                    wrkSpaceRibbonRoot = demoWS.WorkspaceRibbonRoot;
                    //ed.WriteMessage("Added Demo Workspace\n");
                }

                
                //Create the WSRibbonTabSourceReference from RibbonTabSource 
                wsRibTabSrcRef = createWSRibbonTabSrcReference();  
               
                if (wsRibTabSrcRef == null)
                {
                    ed.WriteMessage("Failed to create WSRibbonTabSourceReference\n");
                    return;
                }

                //Get the Tabcollection in the Acme Workspace
                wsRibbonTabCollection = wrkSpaceRibbonRoot.WorkspaceTabs;
                if (wsRibbonTabCollection.Count <= 0)
                {
                    ed.WriteMessage("Demo Workspace does not currently contain any Tabs\n");
                }

                //Get a count of the Ribbons in the ribbon collection in Demo Workspace
                int intialTabCount = wsRibbonTabCollection.Count;

                // Make the parent of the WSRibbonTabSourceReference the Demo Workspace RibbonRoot
                wsRibTabSrcRef.SetParent(wrkSpaceRibbonRoot);

                //Add the WSRibbonTabSourceReference to the Demo Workspace Ribbon Tab collection
                wsRibbonTabCollection.Add(wsRibTabSrcRef);

                //Check the count of the Ribbon Tab collection verifies the new Tab was added 
                if (wsRibbonTabCollection.Count != intialTabCount + 1)
                {
                    ed.WriteMessage("Failed To add WSRibbonTabSourceReference to Demo Workspace");
                    return;
                }
                
                
                //true parameter will make a workspace Demo workspace current
                //after CUIx file is reloaded
                saveCui(true);
            }
            catch (System.Exception es)
            {
                ed.WriteMessage(es.Message);
            }
            finally
            {
                if (wrkSpaceRibbonRoot != null)
                    wrkSpaceRibbonRoot = null;
                if (wsRibTabSrcRef != null)
                    wsRibTabSrcRef = null;
                if (wsRibbonTabCollection != null)
                    wsRibbonTabCollection = null;

            }

        }

        WSRibbonTabSourceReference createWSRibbonTabSrcReference()
        {
            RibbonTabSource ribTabSrc = null;
            RibbonRoot ribbonRoot = null;
            WSRibbonTabSourceReference wsRibTabSrcRef = null;

            ribbonRoot = cs.MenuGroup.RibbonRoot;
            if (ribbonRoot == null)
            {
                ed.WriteMessage("RibbonRoot Not found\n");
                return null;
            }
            //Find the the Tab 
            //ribTabSrc = ribbonRoot.FindTab(strElementId); //Don't know the element id
            //search by name instead, there can be tabs in the collection with the 
            //same name. This example is for demonstration purposes only. Need to use
            //the elementId if at all possible 
            foreach (RibbonTabSource ribTabSrc2 in ribbonRoot.RibbonTabSources)
            {
               // ed.WriteMessage(ribTabSrc2.Name.ToString());
                if (ribTabSrc2.Name != null)
                {
                    if (ribTabSrc2.Name.ToString() == "DemoTab")
                    {
                        ribTabSrc = ribTabSrc2;
                        break;
                    }
                }
            }

            // Make sure the Ribbon Tab is found
            if (ribTabSrc == null)
            {
                ed.WriteMessage("Ribbon Tab Not found\n");
                return null;
            }

            //Create the WSRibbonTabSourceReference using the RibbonTabSource
            wsRibTabSrcRef = WSRibbonTabSourceReference.Create(ribTabSrc);

            //Return the WSRibbonTabSourceReference
            return wsRibTabSrcRef;
        }

        //Runtime Ribbon API and CUIx file API is used in this example
        [CommandMethod("ChangeImageFromMacro")]
        public void ChangeImageFromMacro()
        {
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            try
            {
                Autodesk.Windows.RibbonControl ribControl;
                ribControl = Autodesk.Windows.ComponentManager.Ribbon;

                Autodesk.Windows.RibbonTab ribbonTab;
                ribbonTab = ribControl.ActiveTab;
                ed.WriteMessage(ribbonTab.Name.ToString());

                Autodesk.Windows.RibbonButton ribButton;

                //this works because pline is not in a drop down
                ribButton = (Autodesk.Windows.RibbonButton)ribControl.FindItem("ID_Line", false);

                // Get image from MenuMacro, Change the string to the ElementID of 
                // the MenuMacro that you want to get the image of
                string strElementID = "ID_DimstyleA";
                System.Drawing.Bitmap drwSmallBmp = GetImageFromMacro(strElementID, true);
                System.Drawing.Bitmap drwLargeBmp = GetImageFromMacro(strElementID, false);

                if (drwSmallBmp != null && drwLargeBmp != null)
                {
                    // Save the small bitmap to a MemoryStream
                    MemoryStream strm = new MemoryStream();
                    drwSmallBmp.Save(strm, System.Drawing.Imaging.ImageFormat.Png);

                    // Use the MemoryStream with a BitmapImage
                    BitmapImage bmpImage = new BitmapImage();
                    bmpImage.BeginInit();
                    strm.Seek(0, SeekOrigin.Begin);
                    bmpImage.StreamSource = strm;
                    bmpImage.EndInit();

                    //Use the BitmapImage for small image
                    ribButton.Image = bmpImage;

                    // Save the large bitmap to a MemoryStream
                    strm = new MemoryStream();
                    drwLargeBmp.Save(strm, System.Drawing.Imaging.ImageFormat.Png);

                    // Use the MemoryStream with a BitmapImage
                    bmpImage = new BitmapImage();
                    bmpImage.BeginInit();
                    strm.Seek(0, SeekOrigin.Begin);
                    bmpImage.StreamSource = strm;
                    bmpImage.EndInit();

                    //Use the BitmapImage for large image
                    ribButton.LargeImage = bmpImage;

                    //Change the text
                    ribButton.Text = "Image from Macro";

                    //for testing
                    //ed.WriteMessage(ribButton.Text.ToString());
                }
                else
                {
                    ed.WriteMessage("\nProblem getting image from macro with that ElementID");
                }

            }
            catch (System.Exception ex)
            {
                ed.WriteMessage(ex.Message);
            }
            finally
            {

            }
        }
        public System.Drawing.Bitmap GetImageFromMacro(string strElementID, bool bSmallImg)
        {
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // retrieve the location of, and open the ACAD Main CUI File
            string mainCuiFile = (string)Autodesk.AutoCAD.ApplicationServices.Application.GetSystemVariable("MENUNAME");

            mainCuiFile += ".cuix";
            CustomizationSection cs = new CustomizationSection(mainCuiFile);

            MacroGroup macGroup;
            macGroup = cs.MenuGroup.MacroGroups[0];

            // MenuMacro menuMac;
            foreach (MenuMacro menuMac in macGroup.MenuMacros)
            {
                //for testing
                //  ed.WriteMessage("\n" + menuMac.ElementID.ToString());
                if (menuMac.ElementID.ToString() == strElementID)
                {
                    // for testing
                    //ed.WriteMessage("\n" + menuMac.macro.LargeImage.ToString());
                    //ed.WriteMessage("\n" + menuMac.macro.SmallImage.ToString());
                    if (bSmallImg)
                    {
                        //return the small image
                        System.Drawing.Bitmap drwSmallBmp;
                        drwSmallBmp = menuMac.macro.SmallImageBitmap;
                        return drwSmallBmp;
                    }
                    else
                    {

                        System.Drawing.Bitmap drwLargeBmp;
                        drwLargeBmp = menuMac.macro.LargeImageBitmap;
                        return drwLargeBmp;
                    }

                }

            }
            return null;
        }

        // Save all the open CUI Files that have been modified
        public void saveCui(bool bSetWorkspace)
        {
            Document doc = Application.DocumentManager.MdiActiveDocument;
            // Save Changes to the CUIx file 
            if (cs.IsModified)
                cs.Save();

            // Here we unload and reload the main CUIx file so the changes
            // to the CUIx file can take effect immediately.

            object fd = Application.GetSystemVariable("FILEDIA");
            Application.SetSystemVariable("FILEDIA", 0);

            string flName = cs.CUIFileBaseName;
            RunCommand(doc, "_.CUIUNLOAD " + flName + " ");
            RunCommand(doc, "_.CUILOAD " + flName + " ");
            RunCommand(doc, "_FILEDIA " + fd.ToString() + " ");
            RunCommand(doc, "_.WSCURRENT " + "2D Drafting & Annotation" + "\n");
            if (bSetWorkspace)
            {
                RunCommand(doc, "_.WSCURRENT " + workspaceName + "\n");
                RunCommand(doc, "_.RIBBON ");
            }
           // RunCommand(doc, "_.MENUBAR 1 ");
        }

        private static void RunCommand(Document doc, String cmdString)
        {
            doc.SendStringToExecute(cmdString, false, false, true);
        }
    }

}
